@@ -1,6 +1,6 @@
 package DBIx::Meld::ResultSet;
 BEGIN {
-  $DBIx::Meld::ResultSet::VERSION = '0.04';
+  $DBIx::Meld::ResultSet::VERSION = '0.05';
 }
 use Moose;
 use namespace::autoclean;
@@ -23,6 +23,27 @@ and provides a way to progressively build a SQL query, much like L<DBIx::Class::
 
 =cut
 
+with 'DBIx::Meld::Traits::SQLAbstract';
+
+use Data::Page;
+use Carp qw( croak );
+use Clone qw( clone );
+
+=head1 ATTRIBUTES
+
+=head2 meld
+
+The L<DBIx::Meld> object that this resultset is using.  This attribute provides a
+proxy method to connector so that you can do:
+
+    $resultset->connector->run(sub{  ... });
+
+Instead of:
+
+    $resultset->meld->connector->run(sub{ ... });
+
+=cut
+
 has 'meld' => (
     is       => 'ro',
     isa      => 'DBIx::Meld',
@@ -32,19 +53,103 @@ has 'meld' => (
     )],
 );
 
+=head2 table
+
+The name of the table that this resultset will be using for queries.
+
+=cut
+
 has 'table' => (
     is       => 'ro',
     isa      => 'Str',
     required => 1,
 );
 
+=head2 where
+
+The where clause hash ref to be used when executing queries.
+
+=cut
+
 has 'where' => (
     is      => 'ro',
     isa     => 'HashRef',
     default => sub{ {} },
 );
 
-with 'DBIx::Meld::Traits::SQLAbstract';
+=head2 order_by
+
+An order by value just like L<SQL::Abstract> accepts.
+
+=cut
+
+has 'order_by' => (
+    is  => 'ro',
+    isa => 'Defined',
+);
+
+=head2 rows
+
+The number of rows to return or, if pageing, then number of rows per-page.
+Note that this does not limit the number of rows updated/deleted.
+
+=cut
+
+has 'rows' => (
+    is  => 'ro',
+    isa => 'Int',
+);
+
+=head2 page
+
+The page number.  If set, then this resultset will become paged
+and you will be able to access the pager() attribute.
+
+=cut
+
+has 'page' => (
+    is  => 'ro',
+    isa => 'Int',
+);
+
+=head2 pager
+
+A L<Data::Page> object pre-populated based on page() and rows().  If
+page() has not been specified then trying to access page() will throw
+a fatal error.
+
+The total_entries and last_page methods are proxied from the pager in
+to this class so that you can call:
+
+    print $rs->total_entries();
+
+Instead of:
+
+    print $rs->pager->total_entries();
+
+=cut
+
+has 'pager' => (
+    is         => 'ro',
+    isa        => 'Data::Page',
+    lazy_build => 1,
+    handles => [qw(
+        total_entries
+        last_page
+    )],
+);
+sub _build_pager {
+    my ($self) = @_;
+
+    croak 'pager() can only be called on pageing resultsets' if !$self->page();
+
+    my $pager = Data::Page->new();
+    $pager->total_entries( $self->search({}, {page=>0})->count() );
+    $pager->entries_per_page( $self->rows() || 10 );
+    $pager->current_page( $self->page() );
+
+    return $pager;
+}
 
 =head1 METHODS
 
@@ -61,15 +166,27 @@ resultset's where clause is left unmodified.
 =cut
 
 sub search {
-    my ($self, $where) = @_;
+    my ($self, $where, $clauses) = @_;
 
-    my $new_where = { %{ $self->where() } };
+    $where ||= {};
+    my $new_where = clone( $self->where() );
     map { $new_where->{$_} = $where->{$_} } keys %$where;
 
+    foreach my $clause (qw( order_by rows page )) {
+        next if defined $clauses->{$clause};
+
+        my $value = $self->$clause();
+        next if !defined $value;
+
+        $value = clone( $value) if (ref $value);
+        $clauses->{$clause} = $value;
+    }
+
     return DBIx::Meld::ResultSet->new(
-        meld  => $self->meld(),
-        table => $self->table().
-        where => $new_where,
+        meld    => $self->meld(),
+        table   => $self->table(),
+        where   => $new_where,
+        %$clauses,
     );
 }
 
@@ -81,8 +198,8 @@ sub search {
 =cut
 
 around 'insert' => sub{
-    my ($orig, $self, @args) = @_;
-    return $self->$orig( $self->table(), @args );
+    my ($orig, $self, $values) = @_;
+    return $self->$orig( $self->table(), $values );
 };
 
 =head2 update
@@ -93,8 +210,8 @@ around 'insert' => sub{
 =cut
 
 around 'update' => sub{
-    my ($orig, $self, $fields, @args) = @_;
-    return $self->$orig( $self->table(), $fields, $self->where(), @args );
+    my ($orig, $self, $values) = @_;
+    return $self->$orig( $self->table(), $values, $self->where() );
 };
 
 =head2 delete
@@ -105,7 +222,7 @@ around 'update' => sub{
 
 around 'delete' => sub{
     my ($orig, $self, @args) = @_;
-    return $self->$orig( $self->table(), $self->where(), @args );
+    return $self->$orig( $self->table(), $self->where() );
 };
 
 =head2 array_row
@@ -116,9 +233,21 @@ around 'delete' => sub{
 
 =cut
 
+sub _clause_args {
+    my ($self) = @_;
+    my @args;
+
+    push @args, $self->order_by() || '';
+    return @args if !$self->page();
+
+    push @args, $self->pager->entries_per_page();
+    push @args, $self->pager->skipped();
+    return @args;
+}
+
 around 'array_row' => sub{
-    my ($orig, $self, $fields, @args) = @_;
-    return $self->$orig( $self->table(), $fields, $self->where(), @args );
+    my ($orig, $self, $fields) = @_;
+    return $self->$orig( $self->table(), $fields, $self->where(), $self->_clause_args() );
 };
 
 =head2 hash_row
@@ -129,8 +258,8 @@ around 'array_row' => sub{
 =cut
 
 around 'hash_row' => sub{
-    my ($orig, $self, $fields, @args) = @_;
-    return $self->$orig( $self->table(), $fields, $self->where(), @args );
+    my ($orig, $self, $fields) = @_;
+    return $self->$orig( $self->table(), $fields, $self->where(), $self->_clause_args() );
 };
 
 =head2 array_of_array_rows
@@ -143,8 +272,8 @@ around 'hash_row' => sub{
 =cut
 
 around 'array_of_array_rows' => sub{
-    my ($orig, $self, $fields, @args) = @_;
-    return $self->$orig( $self->table(), $fields, $self->where(), @args );
+    my ($orig, $self, $fields) = @_;
+    return $self->$orig( $self->table(), $fields, $self->where(), $self->_clause_args() );
 };
 
 =head2 array_of_hash_rows
@@ -161,8 +290,8 @@ around 'array_of_hash_rows' => sub{
 =cut
 
 around 'hash_of_hash_rows' => sub{
-    my ($orig, $self, $key, $fields, @args) = @_;
-    return $self->$orig( $key, $self->table(), $fields, $self->where(), @args );
+    my ($orig, $self, $key, $fields) = @_;
+    return $self->$orig( $key, $self->table(), $fields, $self->where(), $self->_clause_args() );
 };
 
 =head2 count
@@ -170,8 +299,9 @@ around 'hash_of_hash_rows' => sub{
 =cut
 
 around 'count' => sub{
-    my ($orig, $self, @args) = @_;
-    return $self->$orig( $self->table(), $self->where(), @args );
+    my ($orig, $self) = @_;
+    return $self->pager->entries_on_this_page() if $self->page();
+    return $self->$orig( $self->table(), $self->where(), $self->_clause_args() );
 };
 
 =head2 column
@@ -179,8 +309,8 @@ around 'count' => sub{
 =cut
 
 around 'column' => sub{
-    my ($orig, $self, $column, @args) = @_;
-    return $self->$orig( $self->table(), $column, $self->where(), @args );
+    my ($orig, $self, $column) = @_;
+    return $self->$orig( $self->table(), $column, $self->where(), $self->_clause_args() );
 };
 
 =head2 select_sth
@@ -188,8 +318,8 @@ around 'column' => sub{
 =cut
 
 around 'select_sth' => sub{
-    my ($orig, $self, $fields, @args) = @_;
-    return $self->$orig( $self->table(), $fields, $self->where(), @args );
+    my ($orig, $self, $fields) = @_;
+    return $self->$orig( $self->table(), $fields, $self->where(), $self->_clause_args() );
 };
 
 =head2 insert_sth
